{
    XviD Stats Analyzer
    Copyright (C) 2005 Yuri Khan
    stats-analyzer at gorodok dot net

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
}

unit GraphOptionsUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, Registry, ComCtrls;

type
  TGraphColors = class(TPersistent)
  private
    FAxes: TColor;
    FBackground: TColor;
    FText: TColor;
    FGridlines: TColor;
    FThreshold: TColor;
    FData: TColor;
    FFiltered: TColor;
    FAverage: TColor;
    procedure ReadColor(Registry: TRegistry; Name: string;
      var Result: TColor);
    procedure WriteColor(Registry: TRegistry; Name: string; Color: TColor);
  public
    constructor Create;
    procedure Assign(Source: TPersistent); override;

    procedure SaveToRegistry(Registry: TRegistry);
    procedure LoadFromRegistry(Registry: TRegistry);

  published
    property Background: TColor read FBackground write FBackground;
    property Axes: TColor read FAxes write FAxes;
    property Text: TColor read FText write FText;
    property Gridlines: TColor read FGridlines write FGridlines;
    property Threshold: TColor read FThreshold write FThreshold;
    property Average: TColor read FAverage write FAverage;
    property Filtered: TColor read FFiltered write FFiltered;
    property Data: TColor read FData write FData;
  end;

  type TBitrateUnit = (buByte, buKbps);

  TGraphOptionsForm = class(TForm)
    OKButton: TButton;
    CancelButton: TButton;
    AxesLabel: TLabel;
    TextLabel: TLabel;
    GridlinesLabel: TLabel;
    BackgroundLabel: TLabel;
    ThresholdLabel: TLabel;
    AverageLabel: TLabel;
    DataLabel: TLabel;
    FilteredLabel: TLabel;
    BackgroundColorBox: TColorBox;
    AxesColorBox: TColorBox;
    TextColorBox: TColorBox;
    GridlinesColorBox: TColorBox;
    ThresholdColorBox: TColorBox;
    AverageColorBox: TColorBox;
    DataColorBox: TColorBox;
    FilteredColorBox: TColorBox;
    BitrateThresholdEdit: TEdit;
    FpsComboBox: TComboBox;
    BitrateThresholdLabel: TLabel;
    FpsLabel: TLabel;
    BitrateUnitComboBox: TComboBox;
    Bevel1: TBevel;
    Bevel2: TBevel;
    procedure OKButtonClick(Sender: TObject);
    procedure BitrateUnitComboBoxChange(Sender: TObject);
  private
    { Private declarations }
    FColors: TGraphColors;
    FFps: Extended;
    FBitrateThreshold: Integer;
    FBitrateUnit: TBitrateUnit;
    function IsValid: Boolean;
    procedure SetBitrateUnit(const Value: TBitrateUnit);
    function ConvertBitrate(source, dest: TBitrateUnit;
      Value: Extended): Extended;
    property BitrateUnit: TBitrateUnit read FBitrateUnit write SetBitrateUnit;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    property Colors: TGraphColors read FColors write FColors;
    property BitrateThreshold: Integer read FBitrateThreshold write FBitrateThreshold;
    property Fps: Extended read FFps write FFps;
    function Execute: Boolean;
  end;

var
  GraphOptionsForm: TGraphOptionsForm;

implementation

{$R *.dfm}

{ TGraphColors }

procedure TGraphColors.Assign(Source: TPersistent);
var Src: TGraphColors;
begin
  if Source is TPersistent then
  begin
    Src := Source as TGraphColors;
    FAxes := Src.FAxes;
    FBackground := Src.FBackground;
    FText := Src.FText;
    FGridlines := Src.FGridlines;
    FThreshold := Src.FThreshold;
    FAverage := Src.FAverage;
    FData := Src.FData;
    FFiltered := Src.FFiltered;
  end
  else
  begin
    inherited;
  end;
end;

constructor TGraphColors.Create;
begin
  FAxes := clBlack;
  FBackground := clWhite;
  FText := clBlack;
  FGridlines := clSilver;
  FThreshold := clRed;
  FAverage := clYellow;
  FData := clBlack;
  FFiltered := clTeal;
end;

procedure TGraphColors.LoadFromRegistry(Registry: TRegistry);
begin
  ReadColor(Registry, 'Axes', FAxes);
  ReadColor(Registry, 'Background', FBackground);
  ReadColor(Registry, 'Text', FText);
  ReadColor(Registry, 'Gridlines', FGridlines);
  ReadColor(Registry, 'Threshold', FThreshold);
  ReadColor(Registry, 'Average', FAverage);
  ReadColor(Registry, 'Data', FData);
  ReadColor(Registry, 'Filtered', FFiltered);
end;

procedure TGraphColors.SaveToRegistry(Registry: TRegistry);
begin
  WriteColor(Registry, 'Axes', FAxes);
  WriteColor(Registry, 'Background', FBackground);
  WriteColor(Registry, 'Text', FText);
  WriteColor(Registry, 'Gridlines', FGridlines);
  WriteColor(Registry, 'Threshold', FThreshold);
  WriteColor(Registry, 'Average', FAverage);
  WriteColor(Registry, 'Data', FData);
  WriteColor(Registry, 'Filtered', FFiltered);
end;

procedure TGraphColors.ReadColor(Registry: TRegistry; Name: string; var Result: TColor);
begin
  try
    Result := StringToColor(Registry.ReadString(Name));
  except
  end;
end;

procedure TGraphColors.WriteColor(Registry: TRegistry; Name: string; Color: TColor);
begin
  try
    Registry.WriteString(Name, ColorToString(Color));
  except
  end;
end;

{ TColorsForm }

constructor TGraphOptionsForm.Create(AOwner: TComponent);
begin
  inherited;
  FColors := TGraphColors.Create;
end;

destructor TGraphOptionsForm.Destroy;
begin
  FreeAndNil(FColors);
  inherited;
end;

function TGraphOptionsForm.Execute: Boolean;
begin
  AxesColorBox.Selected := FColors.Axes;
  BackgroundColorBox.Selected := FColors.Background;
  TextColorBox.Selected := FColors.Text;
  GridlinesColorBox.Selected := FColors.Gridlines;
  ThresholdColorBox.Selected := FColors.Threshold;
  DataColorBox.Selected := FColors.Data;
  FilteredColorBox.Selected := FColors.Filtered;
  AverageColorBox.Selected := FColors.Average;

  BitrateThresholdEdit.Text := FloatToStr(FBitrateThreshold);
  FpsComboBox.Text := FloatToStr(FFps);

  Result := mrOk = ShowModal;

  if Result then
  begin
    FColors.Axes := AxesColorBox.Selected;
    FColors.Background := BackgroundColorBox.Selected;
    FColors.Text := TextColorBox.Selected;
    FColors.Gridlines := GridlinesColorBox.Selected;
    FColors.Threshold := ThresholdColorBox.Selected;
    FColors.Data := DataColorBox.Selected;
    FColors.Filtered := FilteredColorBox.Selected;
    FColors.Average := AverageColorBox.Selected;

    FBitrateThreshold := Round(ConvertBitrate(FBitrateUnit, buByte,
      StrToFloat(BitrateThresholdEdit.Text)));
    FFps := StrToFloat(FpsComboBox.Text);
  end;
end;

procedure TGraphOptionsForm.OKButtonClick(Sender: TObject);
begin
  if not IsValid then
  begin
    ModalResult := mrNone;
  end;
end;

function TGraphOptionsForm.IsValid: Boolean;
begin
  try
    StrToInt(BitrateThresholdEdit.Text);
  except
    on E: Exception do
    begin
      MessageBox(Handle, PChar(E.Message), nil, MB_ICONERROR);
      Result := False;
      Exit;
    end;
  end;

  try
    StrToFloat(FpsComboBox.Text);
  except
    on E: Exception do
    begin
      MessageBox(Handle, PChar(E.Message), nil, MB_ICONERROR);
      Result := False;
      Exit;
    end;
  end;

  Result := True;
end;

procedure TGraphOptionsForm.BitrateUnitComboBoxChange(Sender: TObject);
begin
  BitrateUnit := TBitrateUnit(BitrateUnitComboBox.ItemIndex);
end;

function TGraphOptionsForm.ConvertBitrate(source, dest: TBitrateUnit;
  Value: Extended): Extended;
begin
  if source = dest then
  begin
    Result := Value;
    Exit;
  end;
  if (source = buByte) and (dest = buKbps) then
  begin
    Result{kbit/s} := Value{byte/frame} * StrToFloat(FpsComboBox.Text){frame/s} * 8{bit/byte} / 1000{bit/kbit};
  end
  else if (source = buKbps) and (dest = buByte) then
  begin
    Result{byte/frame} := Value{kbit/s} * 1000{bit/kbit} / (8{bit/byte} * StrToFloat(FpsComboBox.Text){frame/s});
  end
  else
  begin
    raise Exception.Create('incorrect conversion of bitrate units');
  end;
end;

procedure TGraphOptionsForm.SetBitrateUnit(const Value: TBitrateUnit);
begin
  BitrateThresholdEdit.Text := FloatToStr(
    ConvertBitrate(FBitrateUnit, Value,
      StrToFloat(BitrateThresholdEdit.Text)));
  FBitrateUnit := Value;
end;

end.
