{
    XviD Stats Analyzer
    Copyright (C) 2005 Yuri Khan
    stats-analyzer at gorodok dot net

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
}

unit GraphUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, Contnrs, StdCtrls, Registry, ComCtrls, GraphOptionsUnit,
  ToolWin;

type
  TFrameInfo = class
  private
    FFrameType: Char;
    FQuantizer: Integer;
    FIntraBlocks: Integer;
    FFrameSize: Integer;
    FInterBlocks: Integer;
    FHeaderSize: Integer;
    FNotCodedBlocks: Integer;
    FFilteredFrameSize: Integer;
  public
    property FrameType: Char read FFrameType;
    property Quantizer: Integer read FQuantizer;
    property IntraBlocks: Integer read FIntraBlocks;
    property InterBlocks: Integer read FInterBlocks;
    property NotCodedBlocks: Integer read FNotCodedBlocks;
    property FrameSize: Integer read FFrameSize;
    property HeaderSize: Integer read FHeaderSize;

    procedure Read(var f: TextFile);
    property FilteredFrameSize: Integer read FFilteredFrameSize
      write FFilteredFrameSize;
  end;

  TPaintScrollBox = class(TScrollBox)
  private
    FOnPaint: TNotifyEvent;
    FCanvas: TCanvas;
  protected
    procedure WMPaint(var Msg: TWMPaint); message WM_PAINT;
    procedure WMGetDlgCode(var Message: TWMGetDlgCode); message WM_GETDLGCODE;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Canvas: TCanvas read FCanvas;
    property OnPaint: TNotifyEvent read FOnPaint write FOnPaint;
    property OnKeyDown;
    property OnKeyUp;
    property OnKeyPress;
  end;

  TProgressEvent = procedure(Sender: TObject; Progress: Integer) of object;

type
  TFrameIndex = type Integer; // type prefix fx
  TFrameCount = type Integer; // type prefix fw
  TGraphX = type Integer; // type prefix gx
  TGraphWidth = type Integer; // type prefix gw
  TFrameSize = type Integer; // type prefix fs
  TGraphY = type Integer; // type prefix gy
  TGraphHeight = type Integer; // type prefix gh

const
  fx0 = 0;
  gx0 = 0;
  fs0 = 0;

type
  TGraphForm = class;

  TDisplayMode = class
  protected
    procedure DoDrawGridlines(Context: TGraphForm;
      Canvas: TCanvas; cxLeft, cxGraphRight: Integer); virtual; abstract;
    procedure DoDrawTicks(Context: TGraphForm;
      Canvas: TCanvas; cxRight, chGraphHeight: Integer); virtual; abstract;
  public
    procedure DrawGridlines(Context: TGraphForm;
      Canvas: TCanvas; cxLeft, cxGraphRight: Integer);
    procedure DrawTicks(Context: TGraphForm;
      Canvas: TCanvas; cxRight, chGraphHeight: Integer);
  end;

  TDisplayFrameSize = class(TDisplayMode)
  protected
    procedure DoDrawGridlines(Context: TGraphForm;
      Canvas: TCanvas; cxLeft, cxGraphRight: Integer); override;
    procedure DoDrawTicks(Context: TGraphForm;
      Canvas: TCanvas; cxRight, chGraphHeight: Integer); override;
  end;

  TDisplayBitrate = class(TDisplayMode)
  protected
    procedure DoDrawGridlines(Context: TGraphForm;
      Canvas: TCanvas; cxLeft, cxGraphRight: Integer); override;
    procedure DoDrawTicks(Context: TGraphForm;
      Canvas: TCanvas; cxRight, chGraphHeight: Integer); override;
  end;

  TGraphForm = class(TForm)
    GraphPanel: TPanel;
    AxisImage: TImage;
    StatusBar: TStatusBar;
    ToolBar: TToolBar;
    ZoomInButton: TToolButton;
    ZoomOutButton: TToolButton;
    GotoEdit: TEdit;
    ToolbarSeparator1: TToolButton;
    GoButton: TToolButton;
    ToolbarSeparator2: TToolButton;
    OptionsButton: TToolButton;
    GotoLabel: TLabel;
    FrameSizeModeButton: TToolButton;
    ToolbarSeparator3: TToolButton;
    BitrateModeButton: TToolButton;
    procedure GraphPanelResize(Sender: TObject);
    procedure GoButtonClick(Sender: TObject);
    procedure GotoEditKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure OptionsButtonClick(Sender: TObject);
    procedure ZoomInButtonClick(Sender: TObject);
    procedure ZoomOutButtonClick(Sender: TObject);
    procedure FrameSizeModeButtonClick(Sender: TObject);
    procedure BitrateModeButtonClick(Sender: TObject);
  private
    { Private declarations }

    // internal logic
    FFps: Extended;
    FDisplayMode: TDisplayMode;
    FfsMax: TFrameSize;
    FfsTotal: TFrameSize;
    FfsAverage: TFrameSize;
    FfsBitrateThreshold: TFrameSize;
    FFrameIndexZoom: Integer;
    FFrames: TObjectList;

    // selection and position
    FfxSelStart: TFrameIndex;
    FfxSelEnd: TFrameIndex;
    FOnSelectionChange: TNotifyEvent;
    FfxPosition: TFrameIndex;
    FOnPositionChange: TNotifyEvent;

    // loading
    FOnLoadProgress: TProgressEvent;

    // form control
    //FDefaultButton: TButton;
    FPaintScrollBox: TPaintScrollBox;
    FColors: TGraphColors;
    FSelecting: Boolean;

    FPaintColumn: TBitmap;

    // internal logic
    function BitrateFromFs(fs: TFrameSize): Extended;
    function fsFromBitrate(bitrate: Extended): TFrameSize;
    procedure SetDisplayMode(dm: TDisplayMode);
    property DisplayMode: TDisplayMode read FDisplayMode write SetDisplayMode;
    procedure UpdateAverageBitrate;
    procedure SetFsBitrateThreshold(fs: TFrameSize);
    property fsBitrateThreshold: TFrameSize read FfsBitrateThreshold write SetFsBitrateThreshold;
    function gyGraphBottom: TGraphY;
    function ghGraphHeight: TGraphHeight;
    function ghFromFs(fs: TFrameSize): TGraphHeight;
    function cyFromFs(fs: TFrameSize): Integer;

    procedure SetFrameIndexZoom(Value: Integer);
    property FrameIndexZoom: Integer read FFrameIndexZoom write SetFrameIndexZoom;

    function GetGxScrollPos: TGraphX;
    procedure SetGxScrollPos(gx: TGraphX);
    property gxScrollPos: TGraphX read GetGxScrollPos write SetGxScrollPos;

    function GetGwScrollRange: TGraphWidth;
    procedure SetGwScrollRange(gw: TGraphWidth);
    property gwScrollRange: TGraphWidth read GetGwScrollRange write SetGwScrollRange;

    function gxScrollMax: TGraphX;
    function gwWidth: TGraphWidth;
    function gxFromFx(fx: TFrameIndex): TGraphX;
    function gwFromFw(fw: TFrameCount): TGraphWidth;
    function fxFromGx(gx: TGraphX): TFrameIndex;
    function fwFromGw(gw: TGraphWidth): TFrameCount;
    function GetFrames(fx: TFrameIndex): TFrameInfo;
    function fwFrameCount: TFrameCount;
    function fxLast: TFrameIndex;
    procedure AddFrame(fi: TFrameInfo);
    property Frames[fx: TFrameIndex]: TFrameInfo read GetFrames;

    // selection and position
    procedure SetFxSelStart(fx: TFrameIndex);
    procedure SetFxSelEnd(fx: TFrameIndex);
    procedure SetFxPosition(fx: TFrameIndex);
    procedure SetFxPositionSel(fx: TFrameIndex);
    procedure SelectionChange;
    procedure PositionChange;
    procedure InvertSelection(DC: HDC);
    procedure InvertPosition(DC: HDC);

    // form control
    //procedure SetDefaultButton(Value: TButton);
    //property DefaultButton: TButton read FDefaultButton write SetDefaultButton;
    procedure ScrollView;
    procedure PaintScrollBoxKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure PaintScrollBoxMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure PaintScrollBoxMouseMove(Sender: TObject; Shift: TShiftState;
      X, Y: Integer);
    procedure PaintScrollBoxMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure PaintScrollBoxPaint(Sender: TObject);

    procedure UpdateStatusBar(fx: TFrameIndex);
    procedure UpdateGraph;

    procedure DrawGridline(Canvas: TCanvas; fs: TFrameSize; cxLeft, cxGraphRight: Integer);
    procedure DrawTick(Canvas: TCanvas; cxRight: Integer; fs: TFrameSize; Caption: string);

  public
    { Public declarations }

    // construction/destruction
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    // loading
    property OnLoadProgress: TProgressEvent read FOnLoadProgress write FOnLoadProgress;
    procedure LoadFile(Filename: string);

    // selection and position
    property fxSelStart: TFrameIndex read FfxSelStart write SetFxSelStart;
    property fxSelEnd: TFrameIndex read FfxSelEnd write SetFxSelEnd;
    property OnSelectionChange: TNotifyEvent read FOnSelectionChange write FOnSelectionChange;

    property fxPosition: TFrameIndex read FfxPosition write SetFxPosition;
    property OnPositionChange: TNotifyEvent read FOnPositionChange write FOnPositionChange;

    // persistence
    procedure SaveSettings(Registry: TRegistry);
    procedure LoadSettings(Registry: TRegistry);
  end;

var
  dmFrameSize, dmBitrate: TDisplayMode;
  GraphForm: TGraphForm;

implementation

uses Math;

{$R *.dfm}

function NormDist(x, mu, sigma: Extended): Extended;
begin
  Result := exp(-(sqr(x - mu) / (2 * sqr(sigma)) )) / (sqrt(2 * Pi) * sigma);
end;

function LogRoundUp(n: Extended): Integer;
begin
  case Ceil(n) of
    1: Result := 1;
    2: Result := 2;
    3..5: Result := 5;
    6..10: Result := 10;
    11..20: Result := 20;
  else
    Result := Trunc(IntPower(10, Trunc(Log10(n - 1)) + 1));
    case Trunc(Result / n) of
      1: ; // do nothing
      2..4: Result := Result div 2;
      5..9: Result := Result div 5;
    else
      Result := 0; // should not happen, will trigger a zero divide
    end
  end;
end;

{ TFrameInfo }

procedure TFrameInfo.Read(var f: TextFile);
begin
  ReadLn(f, FFrameType, FQuantizer,
    FIntraBlocks, FInterBlocks, FNotCodedBlocks,
    FFrameSize, FHeaderSize);
end;

{ TPaintScrollBox }

procedure TPaintScrollBox.WMPaint(var Msg: TWMPaint);
begin
  inherited;
  FCanvas := TCanvas.Create;
  try
    FCanvas.Handle := GetDC(Handle);
    try
      if Assigned(FOnPaint) then
      begin
        FOnPaint(Self);
      end;
    finally
      ReleaseDC(Handle, FCanvas.Handle);
    end;
  finally
    FCanvas.Free;
  end;
end;

procedure TPaintScrollBox.WMGetDlgCode(var Message: TWMGetDlgCode);
begin
  Message.Result := DLGC_WANTARROWS;
end;

constructor TPaintScrollBox.Create(AOwner: TComponent);
begin
  inherited;
  ControlStyle := ControlStyle - [csAcceptsControls];
end;

{ TGraphForm }

// internal logic

function TGraphForm.BitrateFromFs(fs: TFrameSize): Extended;
begin
  Result{kbit/s} := FFps{frames/s} * fs{byte/frame} * 8{bit/byte} /
    1000{bit/kbit};
end;

function TGraphForm.fsFromBitrate(bitrate: Extended): TFrameSize;
begin
  Result{byte/frame} := Round(bitrate{kbit/s} * 1000{bit/kbit} /
    (FFps{frames/s} * 8{bit/byte}));
end;

procedure TGraphForm.SetDisplayMode(dm: TDisplayMode);
begin
  FDisplayMode := dm;
  UpdateGraph;
end;

procedure TGraphForm.UpdateAverageBitrate;
begin
  if fwFrameCount <> 0 then
  begin
    StatusBar.Panels[8].Text := Format('Average bitrate: %f kbps', [
      BitrateFromFs(FfsTotal div fwFrameCount) ]);
  end
  else
  begin
    StatusBar.Panels[8].Text := '';
  end;
end;

procedure TGraphForm.SetFsBitrateThreshold(fs: TFrameSize);
begin
  FfsBitrateThreshold := fs;
end;

function TGraphForm.gyGraphBottom: TGraphY;
begin
  Result := AxisImage.Height - 50;
end;

function TGraphForm.ghGraphHeight: TGraphHeight;
begin
  Result :=AxisImage.Height - 100;
end;

function TGraphForm.ghFromFs(fs: TFrameSize): TGraphHeight;
begin
  Result := fs * ghGraphHeight div FfsMax;
end;

function TGraphForm.cyFromFs(fs: TFrameSize): Integer;
begin
  Result := gyGraphBottom - ghFromFs(fs);
end;

procedure TGraphForm.SetFrameIndexZoom(Value: Integer);
var gwOldX: TGraphWidth;
begin
  gwOldX := gxFromFx(FfxPosition) - gxScrollPos;
  FFrameIndexZoom := Value;

  gwScrollRange := gxFromFx(fx0 + fwFrameCount) - gxFromFx(fx0);
  if fwFrameCount <> 0 then
  begin
    FPaintScrollBox.HorzScrollBar.ThumbSize :=
      (FPaintScrollBox.Width - 2 * FPaintScrollBox.HorzScrollBar.ButtonSize) *
      FPaintScrollBox.Width div FPaintScrollBox.HorzScrollBar.Range;
  end
  else begin
    FPaintScrollBox.HorzScrollBar.ThumbSize :=
      (FPaintScrollBox.Width - 2 * FPaintScrollBox.HorzScrollBar.ButtonSize);
  end;
  gxScrollPos := gxFromFx(FfxPosition) - gwOldX;

  UpdateGraph;
end;

function TGraphForm.GetGxScrollPos: TGraphX;
begin
  Result := FPaintScrollBox.HorzScrollBar.Position;
end;

procedure TGraphForm.SetGxScrollPos(gx: TGraphX);
begin
  FPaintScrollBox.HorzScrollBar.Position := gx;
end;

function TGraphForm.GetGwScrollRange: TGraphWidth;
begin
  Result := FPaintScrollBox.HorzScrollBar.Range;
end;

procedure TGraphForm.SetGwScrollRange(gw: TGraphWidth);
begin
  FPaintScrollBox.HorzScrollBar.Range := gw;
end;

function TGraphForm.gxScrollMax: TGraphX;
begin
  Result := gx0 + gwScrollRange - gwWidth;
end;

function TGraphForm.gwWidth: TGraphWidth;
begin
  Result := FPaintScrollBox.Width;
end;

function TGraphForm.gxFromFx(fx: TFrameIndex): TGraphX;
begin
  Result := fx div FFrameIndexZoom;
end;

function TGraphForm.gwFromFw(fw: TFrameCount): TGraphWidth;
begin
  Result := fw div FFrameIndexZoom;
end;

function TGraphForm.fxFromGx(gx: TGraphX): TFrameIndex;
begin
  Result := gx * FFrameIndexZoom;
end;

function TGraphForm.fwFromGw(gw: TGraphWidth): TFrameCount;
begin
  Result := gw * FFrameIndexZoom;
end;

function TGraphForm.GetFrames(fx: TFrameIndex): TFrameInfo;
begin
  Result := FFrames[fx] as TFrameInfo;
end;

function TGraphForm.fwFrameCount: TFrameCount;
begin
  Result := FFrames.Count;
end;

function TGraphForm.fxLast: TFrameIndex;
begin
  Result := fx0 + fwFrameCount - 1;
end;

procedure TGraphForm.AddFrame(fi: TFrameInfo);
begin
  FFrames.Add(fi);
end;

// selection and position

procedure TGraphForm.SetFxSelStart(fx: TFrameIndex);
begin
  fx := Max(fx0, Min(fxLast, fx));

  FfxSelStart := fx;
  FPaintScrollBox.Invalidate;
end;

procedure TGraphForm.SetFxSelEnd(fx: TFrameIndex);
begin
  fx := Max(fx0, Min(fx0 + fwFrameCount, fx));

  FfxSelEnd := fx;
  FPaintScrollBox.Invalidate;
end;

procedure TGraphForm.SetFxPosition(fx: TFrameIndex);
var DC: HDC;
begin
  fx := Max(fx0, Min(fxLast, fx));
  if fx = FfxPosition then Exit;

  DC := GetDC(FPaintScrollBox.Handle);
  try
    InvertPosition(DC);
    FfxPosition := fx;
    InvertPosition(DC);
  finally
    ReleaseDC(FPaintScrollBox.Handle, DC);
  end;

  ScrollView;
  UpdateStatusBar(fx);
  PositionChange;
end;

procedure TGraphForm.SetFxPositionSel(fx: TFrameIndex);
var DC: HDC;
begin
  fx := Max(0, Min(fxLast, fx));

  DC := GetDC(FPaintScrollBox.Handle);
  try
    InvertPosition(DC);

    FfxPosition := fx;
    FfxSelStart := fx;
    FfxSelEnd := fx;

    InvertPosition(DC);
  finally
    ReleaseDC(FPaintScrollBox.Handle, DC);
  end;

  UpdateStatusBar(fx);
  PositionChange;
end;

procedure TGraphForm.SelectionChange;
begin
  if Assigned(FOnSelectionChange) then
  begin
    FOnSelectionChange(Self);
  end;
end;

procedure TGraphForm.PositionChange;
begin
  if Assigned(FOnPositionChange) then
  begin
    FOnPositionChange(Self);
  end;
end;

procedure TGraphForm.InvertSelection(DC: HDC);
begin
  InvertRect(DC, Rect(
    gxFromFx(FfxSelStart) - gxScrollPos,
    gyGraphBottom - ghFromFs(FfsMax),
    gxFromFx(FfxSelEnd) - gxScrollPos,
    gyGraphBottom));
end;

procedure TGraphForm.InvertPosition(DC: HDC);
begin
  InvertRect(DC, Rect(
    gxFromFx(FfxPosition) - gxScrollPos,
    gyGraphBottom - ghFromFs(FfsMax),
    gxFromFx(FfxPosition) - gxScrollPos + 1,
    gyGraphBottom));
end;

// form control

//procedure TGraphForm.SetDefaultButton(Value: TButton);
//begin
//  if Assigned(FDefaultButton) then
//  begin
//    FDefaultButton.Default := False;
//  end;
//  FDefaultButton := Value;
//  if Assigned(FDefaultButton) then
//  begin
//    FDefaultButton.Default := True;
//  end;
//end;

procedure TGraphForm.ScrollView;
begin
  if (gxFromFx(FfxPosition) < gxScrollPos)
  or (gxFromFx(FfxPosition) >= gxScrollPos + gwWidth) then
  begin
    gxScrollPos := Max(gx0, Min(gxScrollMax,
      gxFromFx(FfxPosition) - gwWidth div 2));
  end;
end;

procedure TGraphForm.PaintScrollBoxKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
const
  VK_OEM_4 = $DB;
  VK_OEM_6 = $DD;
begin
  case Key of
    VK_LEFT:
      if [ssCtrl] = Shift then
      begin
        fxPosition := 0;
        Key := 0;
      end
      else if [ssShift] = Shift then
      begin
        fxPosition := fxPosition - 10;
        Key := 0;
      end
      else if [] = Shift then
      begin
        fxPosition := fxPosition - 1;
        Key := 0;
      end;
    VK_RIGHT:
      if [ssCtrl] = Shift then
      begin
        fxPosition := fxLast;
        Key := 0;
      end
      else if [ssShift] = Shift then
      begin
        fxPosition := fxPosition + 10;
        Key := 0;
      end
      else if [] = Shift then
      begin
        fxPosition := fxPosition + 1;
        Key := 0;
      end;
    VK_PRIOR:
      if [ssCtrl] = Shift then
      begin
        fxPosition := fxFromGx(gxScrollPos);
      end
      else if [] = Shift then
      begin
        gxScrollPos := gxScrollPos - gwWidth;
        fxPosition := fxPosition - fwFromGw(gwWidth);
      end;
    VK_NEXT:
      if [ssCtrl] = Shift then
      begin
        fxPosition := fxFromGx(gxScrollPos + gwWidth) - 1;
      end
      else if [] = Shift then
      begin
        gxScrollPos := gxScrollPos + gwWidth;
        fxPosition := fxPosition + fwFromGw(gwWidth);
      end;
    VK_OEM_4: // [
      begin
        fxPosition := fxSelStart;
      end;
    VK_OEM_6: // ]
      begin
        fxPosition := fxSelEnd;
      end;
    VK_HOME:
      begin
        fxSelStart := fxPosition;
        SelectionChange;
      end;
    VK_END:
      begin
        fxSelEnd := fxPosition;
        SelectionChange;
      end;

    VK_ADD:
      begin
        ZoomInButton.Click;
      end;
    VK_SUBTRACT:
      begin
        ZoomOutButton.Click;
      end;
  end;
end;

procedure TGraphForm.PaintScrollBoxMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if (Button = mbLeft)
  and (X >= 0)
  and (X < gwFromFw(fwFrameCount)) then
  begin
    FfxSelStart := fxFromGx(gxScrollPos + TGraphWidth(X));
    fxSelEnd := FfxSelStart;
    FSelecting := True;
  end;
end;

procedure TGraphForm.PaintScrollBoxMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
var DC: HDC;
begin
  if FSelecting then
  begin
    DC := GetDC(FPaintScrollBox.Handle);
    try
      InvertSelection(DC); // Clear selection
      FfxSelEnd := Max(fx0, Min(fxLast + 1,
        fxFromGx(gxScrollPos + TGraphWidth(X))));
      UpdateStatusBar(FfxSelEnd);
      InvertSelection(DC); // Draw selection
    finally
      ReleaseDC(FPaintScrollBox.Handle, DC);
    end;
  end;
end;

procedure TGraphForm.PaintScrollBoxMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var DC: HDC;
begin
  if FSelecting and (Button = mbLeft) then
  begin
    DC := GetDC(FPaintScrollBox.Handle);
    try
      InvertSelection(DC); // Clear selection
    finally
      ReleaseDC(FPaintScrollBox.Handle, DC);
    end;
    fxSelEnd := Max(fx0, Min(fxLast + 1,
      fxFromGx(gxScrollPos + TGraphWidth(X))));
    fxPosition := FfxSelStart;

    FSelecting := False;
    SelectionChange;
  end;
end;

procedure TGraphForm.PaintScrollBoxPaint(Sender: TObject);
  var cyGraphTop, cyGraphBottom, chGraphHeight, chHeight, cyBottom,
    cxRight, cxGraphRight, fx, fxTickUnit, cx: Integer;
  const
    cxLeft = 0;
    cyTop = 0;

  function cxFromFx(fx: TFrameIndex): Integer;
  begin
    Result := cxLeft + gxFromFx(fx) - gxScrollPos;
  end;

  function fxFromCx(cx: Integer): TFrameIndex;
  begin
    Result := fxFromGx(gxScrollPos + cx - cxLeft)
  end;

  function fsAverageFilteredIn(fxBegin, fxEnd: TFrameIndex): TFrameSize;
  var fx: TFrameIndex; fsSum: TFrameSize;
  begin
    fsSum := 0;
    fxBegin := Max(fx0, fxBegin);
    fxEnd := Min(fxLast + 1, fxEnd);
    for fx := fxBegin to fxEnd - 1 do
    begin
      Inc(fsSum, Frames[fx].FilteredFrameSize);
    end;
    Result := fsSum div (fxEnd - fxBegin);
  end;

  function fsAverageIn(fxBegin, fxEnd: TFrameIndex): TFrameSize;
  var fx: TFrameIndex; fsSum: TFrameSize;
  begin
    fsSum := 0;
    fxBegin := Max(fx0, fxBegin);
    fxEnd := Min(fxLast + 1, fxEnd);
    for fx := fxBegin to fxEnd - 1 do
    begin
      Inc(fsSum, Frames[fx].FrameSize);
    end;
    Result := fsSum div (fxEnd - fxBegin);
  end;

  procedure DrawSample(Canvas: TCanvas; cx: Integer);
  begin
    Canvas.Pen.Color := FColors.Filtered;
    Canvas.MoveTo(cx, cyGraphBottom);
    Canvas.LineTo(cx, cyFromFs(
      fsAverageFilteredIn(fxFromCx(cx), fxFromCx(cx + 1))));

    Canvas.Pen.Color := FColors.Data;
    Canvas.MoveTo(cx, cyGraphBottom);
    Canvas.LineTo(cx, cyFromFs(
      fsAverageIn(fxFromCx(cx), fxFromCx(cx + 1))));
  end;

begin
  cyGraphBottom := gyGraphBottom;
  chGraphHeight := ghGraphHeight;
  cyGraphTop := cyGraphBottom - chGraphHeight;
  chHeight := FPaintScrollBox.Height;
  cyBottom := cyTop + chHeight;
  cxRight := gwWidth;
  cxGraphRight := Min(gwWidth, gwFromFw(fwFrameCount));

  with FPaintScrollBox.Canvas do
  begin
    // Clear
    Brush.Color := FColors.Background;
    Brush.Style := bsSolid;
    FillRect(Rect(cxLeft, cyTop, cxRight, cyBottom));

    // Axis
    Pen.Color := FColors.Axes;
    Pen.Style := psSolid;
    Pen.Width := 1;
    MoveTo(cxLeft, cyGraphBottom);
    LineTo(cxGraphRight, cyGraphBottom);

    // Size gridlines
    Pen.Color := FColors.Gridlines;
    FDisplayMode.DrawGridlines(Self, FPaintScrollBox.Canvas, cxLeft, cxGraphRight);

    // Frame# gridlines and ticks
    Pen.Color := FColors.Gridlines;
    Font.Name := 'MS Shell Dlg';
    Font.Size := 10;
    Font.Color := FColors.Text;
    SetTextAlign(Handle, TA_LEFT or TA_TOP);
    fxTickUnit := LogRoundUp(Max(1, fwFrameCount * 32.0 / Max(1, gwScrollRange)));
    fx := fxFromGx(gxScrollPos) div fxTickUnit * fxTickUnit;
    while fx < Min(fxLast + 1, fxFromGx(gxScrollPos + gwWidth)) do
    begin
      MoveTo(cxFromFx(fx), cyGraphBottom - 1);
      LineTo(cxFromFx(fx), cyGraphTop);
      TextOut(cxFromFx(fx), cyGraphBottom + 5, IntToStr(fx));

      Inc(fx, fxTickUnit);
    end;

    // Data
    for cx := cxLeft to cxGraphRight - 1 do
    begin
      DrawSample(FPaintScrollBox.Canvas, cx);
    end;

    // Threshold line
    Pen.Color := FColors.Threshold;
    MoveTo(cxLeft, cyFromFs(FfsBitrateThreshold));
    LineTo(cxGraphRight, cyFromFs(FfsBitrateThreshold));

    // Average line
    Pen.Color := FColors.Average;
    MoveTo(cxLeft, cyFromFs(FfsAverage));
    LineTo(cxGraphRight, cyFromFs(FfsAverage));

    // Selection
    InvertSelection(Handle);

    // Position
    InvertPosition(Handle);
  end;
end;

procedure TGraphForm.UpdateStatusBar(fx: TFrameIndex);
begin
  StatusBar.Panels[0].Text := IntToStr(fx);
  if (0 <= fx) and (fx <= fxLast) then
  begin
    with Frames[fx] do
    begin
      StatusBar.Panels[1].Text := UpperCase(FrameType);
      StatusBar.Panels[2].Text := IntToStr(Quantizer);
      StatusBar.Panels[3].Text := IntToStr(IntraBlocks);
      StatusBar.Panels[4].Text := IntToStr(InterBlocks);
      StatusBar.Panels[5].Text := IntToStr(NotCodedBlocks);
      StatusBar.Panels[6].Text := IntToStr(FrameSize);
      StatusBar.Panels[7].Text := IntToStr(HeaderSize);
    end;
  end;
end;

procedure TGraphForm.UpdateGraph;
var cxRight, cyGraphBottom, cyGraphTop, chGraphHeight: Integer;
begin
  if not Assigned(AxisImage.Picture.Bitmap) then
    AxisImage.Picture.Bitmap := TBitmap.Create;

  AxisImage.Picture.Bitmap.Width := AxisImage.Width;
  AxisImage.Picture.Bitmap.Height := AxisImage.Height;

  gwScrollRange := gwFromFw(fwFrameCount);

  with AxisImage.Picture.Bitmap, Canvas do
  begin
    cxRight := Width - 1;
    cyGraphBottom := Height - 50;
    cyGraphTop := 50;
    chGraphHeight := cyGraphBottom - cyGraphTop;

    // Background
    Brush.Color := FColors.Background;
    Brush.Style := bsSolid;
    FillRect(ClipRect);

    // Axis
    Pen.Color := FColors.Axes;
    Pen.Style := psSolid;
    Pen.Width := 1;
    MoveTo(cxRight, cyGraphBottom);
    LineTo(cxRight, cyGraphTop);

    // Ticks
    Font.Name := 'MS Shell Dlg';
    Font.Size := 10;
    Font.Color := FColors.Text;
    SetTextAlign(Handle, TA_RIGHT or TA_BOTTOM);

    FDisplayMode.DrawTicks(Self, AxisImage.Picture.Bitmap.Canvas,
      cxRight, chGraphHeight);
  end;

  FPaintScrollBox.Invalidate;
end;

procedure TGraphForm.DrawGridline(Canvas: TCanvas; fs: TFrameSize;
  cxLeft, cxGraphRight: Integer);
begin
  Canvas.MoveTo(cxLeft, cyFromFs(fs));
  Canvas.LineTo(cxGraphRight, cyFromFs(fs));
end;

procedure TGraphForm.DrawTick(Canvas: TCanvas; cxRight: Integer;
  fs: TFrameSize; Caption: string);
begin
  Canvas.MoveTo(cxRight, cyFromFs(fs));
  Canvas.LineTo(cxRight - 2, cyFromFs(fs));
  Canvas.TextOut(cxRight - 5, cyFromFs(fs), Caption);
end;

// construction/destruction

constructor TGraphForm.Create(AOwner: TComponent);
begin
  inherited;
  FFrames := TObjectList.Create;
  FfsMax := 1;
  FFps := 23.976;
  FFrameIndexZoom := 1;
  FColors := TGraphColors.Create;

  FDisplayMode := dmFrameSize;

  FPaintColumn := TBitmap.Create;

  FPaintScrollBox := TPaintScrollBox.Create(Self);
  FPaintScrollBox.Parent := GraphPanel;
  FPaintScrollBox.Align := alClient;
  FPaintScrollBox.Visible := True;
  FPaintScrollBox.BorderStyle := bsNone;
  FPaintScrollBox.TabStop := True;
  FPaintScrollBox.HorzScrollBar.Smooth := True;
  FPaintScrollBox.HorzScrollBar.Tracking := True;
  FPaintScrollBox.OnPaint := PaintScrollBoxPaint;
  FPaintScrollBox.OnMouseDown := PaintScrollBoxMouseDown;
  FPaintScrollBox.OnMouseMove := PaintScrollBoxMouseMove;
  FPaintScrollBox.OnMouseUp := PaintScrollBoxMouseUp;
  FPaintScrollBox.OnKeyDown := PaintScrollBoxKeyDown;
end;

destructor TGraphForm.Destroy;
begin
  FreeAndNil(FPaintColumn); 
  FreeAndNil(FFrames);
  FreeAndNil(FColors);
  inherited;
end;

// loading

procedure TGraphForm.LoadFile(Filename: string);
const sigma = 5;
var f: TextFile;
  frameInfo: TFrameInfo;
  fx, fw, pct, oldPct: Integer;
  fsSum, denom: Extended;
begin
  FileMode := fmOpenRead or fmShareDenyNone;
  AssignFile(f, Filename);
  Reset(f);
  try
    ReadLn(f); // banner
    ReadLn(f); // banner
    ReadLn(f); // empty line

    FfsMax := fs0;
    FFrames.Clear;
    FfsTotal := fs0;
    while not EOF(f) do
    begin
      frameInfo := TFrameInfo.Create;
      frameInfo.Read(f);
      AddFrame(frameInfo);
      Inc(FfsTotal, frameInfo.FrameSize);
      FfsMax := Max(FfsMax, frameInfo.FrameSize);
    end;

    if fwFrameCount <> 0 then
    begin
      FfsAverage := FfsTotal div fwFrameCount;
    end
    else
    begin
      FfsAverage := {arbitrary}10000;
    end;
    fsBitrateThreshold := Round(1.5 * FfsTotal / fwFrameCount);
    UpdateAverageBitrate;

    oldPct := -1;
    for fx := fx0 to fxLast do
    begin
      if Assigned(FOnLoadProgress) then
      begin
        pct := (fx - fx0) * 100 div fwFrameCount;
        if oldPct <> pct then
        begin
          FOnLoadProgress(Self, pct);
          oldPct := pct;
        end;
      end;

      fsSum := 0;
      denom := 0;
      for fw := Max(-3 * sigma, fx0 - fx)
      to Min(3 * sigma, fxLast - fx) do
      begin
        fsSum := fsSum + NormDist(fw, 0, sigma) * Frames[fx + fw].FrameSize;
        denom := denom + NormDist(fw, 0, sigma);
      end;
      Frames[fx].FilteredFrameSize := Round(fsSum / denom);
    end;

    if Assigned(FOnLoadProgress) then
    begin
      FOnLoadProgress(Self, 100);
    end;

    UpdateGraph;
  finally
    CloseFile(f);
  end;
  fxPosition := fx0;
  UpdateStatusBar(fx0);
end;

// persistence

procedure TGraphForm.SaveSettings(Registry: TRegistry);
begin
  try Registry.WriteInteger('Left', Left); except end;
  try Registry.WriteInteger('Top', Top); except end;
  try Registry.WriteInteger('Width', Width); except end;
  try Registry.WriteInteger('Height', Height); except end;
  try Registry.WriteFloat('FPS', FFps); except end;
  FColors.SaveToRegistry(Registry);
end;

procedure TGraphForm.LoadSettings(Registry: TRegistry);
begin
  try Left := Registry.ReadInteger('Left'); except end;
  try Top := Registry.ReadInteger('Top'); except end;
  try Width := Registry.ReadInteger('Width'); except end;
  try Height := Registry.ReadInteger('Height'); except end;
  try FFps := Registry.ReadFloat('FPS'); except end;
  FColors.LoadFromRegistry(Registry);
end;

// Event handlers

procedure TGraphForm.GraphPanelResize(Sender: TObject);
begin
  UpdateGraph;
end;

procedure TGraphForm.GoButtonClick(Sender: TObject);
var fx: Integer;
begin
  fx := Max(fx0, Min(fxLast, StrToInt(GotoEdit.Text)));
  SetFxPositionSel(fx);
  SelectionChange;
end;

procedure TGraphForm.GotoEditKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_RETURN then
  begin
    GoButton.Click;
  end;
end;

procedure TGraphForm.OptionsButtonClick(Sender: TObject);
begin
  with TGraphOptionsForm.Create(Self) do
  try
    Colors.Assign(FColors);
    BitrateThreshold := FfsBitrateThreshold;
    Fps := FFps;
    if Execute then
    begin
      FColors.Assign(Colors);
      FfsBitrateThreshold := BitrateThreshold;
      FFps := Fps;
      UpdateGraph;
      UpdateAverageBitrate;
    end;
  finally
    Free;
  end;
end;

procedure TGraphForm.ZoomInButtonClick(Sender: TObject);
begin
  FrameIndexZoom := Max(1, FrameIndexZoom div 2);
end;

procedure TGraphForm.ZoomOutButtonClick(Sender: TObject);
begin
  FrameIndexZoom := Max(1, Min(fwFrameCount div 2, FrameIndexZoom) * 2);
end;

procedure TGraphForm.FrameSizeModeButtonClick(Sender: TObject);
begin
  DisplayMode := dmFrameSize;
end;

procedure TGraphForm.BitrateModeButtonClick(Sender: TObject);
begin
  DisplayMode := dmBitrate;
end;

{ TDisplayFrameSize }

procedure TDisplayFrameSize.DoDrawGridlines(Context: TGraphForm;
  Canvas: TCanvas; cxLeft, cxGraphRight: Integer);
var fs: TFrameSize;
begin
  fs := fs0;
  while fs <= Context.FfsMax do
  begin
    Context.DrawGridline(Canvas, fs, cxLeft, cxGraphRight);
    Inc(fs, 10000);
  end;
end;

procedure TDisplayFrameSize.DoDrawTicks(Context: TGraphForm;
  Canvas: TCanvas; cxRight, chGraphHeight: Integer);
var fsTickUnit, fs: TFrameSize;
begin
  fsTickUnit := LogRoundUp(Context.FfsMax * 32.0 / chGraphHeight);
  fs := fs0;
  while fs < Context.FfsMax do
  begin
    Context.DrawTick(Canvas, cxRight, fs, IntToStr(fs));
    Inc(fs, fsTickUnit);
  end;
end;

{ TDisplayBitrate }

procedure TDisplayBitrate.DoDrawGridlines(Context: TGraphForm;
  Canvas: TCanvas; cxLeft, cxGraphRight: Integer);
var bitrate: Extended;
begin
  bitrate := 0.0;
  while bitrate <= Context.BitrateFromFs(Context.FfsMax) do
  begin
    Context.DrawGridline(Canvas, Context.fsFromBitrate(bitrate), cxLeft, cxGraphRight);
    bitrate := bitrate + 1000.0;
  end;
end;

procedure TDisplayBitrate.DoDrawTicks(Context: TGraphForm;
  Canvas: TCanvas; cxRight, chGraphHeight: Integer);
var bitrateTickUnit, bitrate: Extended;
begin
  bitrateTickUnit := LogRoundUp(Context.BitrateFromFs(Context.FfsMax) * 32.0 /
    chGraphHeight);
  bitrate := Context.BitrateFromFs(fs0);
  while bitrate < Context.BitrateFromFs(Context.FfsMax) do
  begin
    Context.DrawTick(Canvas, cxRight, Context.fsFromBitrate(bitrate), FloatToStr(bitrate));
    bitrate := bitrate + bitrateTickUnit;
  end;
end;

{ TDisplayMode }

procedure TDisplayMode.DrawGridlines(Context: TGraphForm; Canvas: TCanvas;
  cxLeft, cxGraphRight: Integer);
begin
  DoDrawGridlines(Context, Canvas, cxLeft, cxGraphRight);
end;

procedure TDisplayMode.DrawTicks(Context: TGraphForm; Canvas: TCanvas;
  cxRight, chGraphHeight: Integer);
begin
  DoDrawTicks(Context, Canvas, cxRight, chGraphHeight);
end;

initialization
  dmFrameSize := TDisplayFrameSize.Create;
  dmBitrate := TDisplayBitrate.Create;
finalization
  FreeAndNil(dmBitrate);
  FreeAndNil(dmFrameSize);
end.

