{
    XviD Stats Analyzer
    Copyright (C) 2005 Yuri Khan
    stats-analyzer at gorodok dot net

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
}

unit VideoPlayerUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, MPlayer, ExtCtrls, ComCtrls, ToolWin, Registry, StdCtrls;

type
  TVideoPlayerForm = class(TForm)
    DisplayPanel: TPanel;
    MediaPlayer: TMediaPlayer;
    ToolBar: TToolBar;
    PlayButton: TToolButton;
    PauseButton: TToolButton;
    StopButton: TToolButton;
    Back10Button: TToolButton;
    BackButton: TToolButton;
    NextButton: TToolButton;
    Next10Button: TToolButton;
    ToolButton1: TToolButton;
    Timer: TTimer;
    procedure PlayButtonClick(Sender: TObject);
    procedure PauseButtonClick(Sender: TObject);
    procedure StopButtonClick(Sender: TObject);
    procedure Back10ButtonClick(Sender: TObject);
    procedure BackButtonClick(Sender: TObject);
    procedure NextButtonClick(Sender: TObject);
    procedure Next10ButtonClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure TimerTimer(Sender: TObject);
  private
    FSelStart: Integer;
    FSelEnd: Integer;
    FPosition: Integer;
    FOldPosition: Integer;
    FOnPositionChange: TNotifyEvent;
    FPlayerReady: Boolean;
    procedure SetSelEnd(Value: Integer);
    procedure SetSelStart(Value: Integer);
    procedure SetPosition(Value: Integer);
    procedure PositionChange;
    { Private declarations }
  public
    { Public declarations }

    procedure LoadFile(Filename: string);

    procedure Pause;
    property SelStart: Integer read FSelStart write SetSelStart;
    property SelEnd: Integer read FSelEnd write SetSelEnd;

    property Position: Integer read FPosition write SetPosition;
    property OnPositionChange: TNotifyEvent read FOnPositionChange write FOnPositionChange;

    procedure SaveSettings(Registry: TRegistry);
    procedure LoadSettings(Registry: TRegistry);
  end;

var
  VideoPlayerForm: TVideoPlayerForm;

implementation

uses Math;

{$R *.dfm}

{ TVideoPlayerForm }

procedure TVideoPlayerForm.LoadFile(Filename: string);
begin
  MediaPlayer.FileName := Filename;
  MediaPlayer.Open;
  Timer.Enabled := True;
  FPlayerReady := True;

  SetBounds(Left, Top,
    Width - DisplayPanel.ClientWidth + MediaPlayer.DisplayRect.Right - MediaPlayer.DisplayRect.Left,
    Height - DisplayPanel.ClientHeight + MediaPlayer.DisplayRect.Bottom - MediaPlayer.DisplayRect.Top);
end;

procedure TVideoPlayerForm.PlayButtonClick(Sender: TObject);
begin
  if FPlayerReady then
  begin
    MediaPlayer.Play;
    Timer.Enabled := True;
  end;
end;

procedure TVideoPlayerForm.PauseButtonClick(Sender: TObject);
begin
  if FPlayerReady then
  begin
    MediaPlayer.Pause;
    Timer.Enabled := True;
  end;
end;

procedure TVideoPlayerForm.StopButtonClick(Sender: TObject);
begin
  if FPlayerReady then
  begin
    MediaPlayer.Stop;
  end;
end;

procedure TVideoPlayerForm.Back10ButtonClick(Sender: TObject);
begin
  Position := Position - 10;
end;

procedure TVideoPlayerForm.BackButtonClick(Sender: TObject);
begin
  Position := Position - 1;
end;

procedure TVideoPlayerForm.NextButtonClick(Sender: TObject);
begin
  Position := Position + 1;
end;

procedure TVideoPlayerForm.Next10ButtonClick(Sender: TObject);
begin
  Position := Position + 10;
end;

procedure TVideoPlayerForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
  const
    VK_OEM_4 = $DB;
    VK_OEM_6 = $DD;
begin
  if FPlayerReady then
  begin
    case Key of
      VK_SPACE:
      begin
        MediaPlayer.Pause;
        Key := 0;
      end;
      VK_LEFT:
        if [ssCtrl] = Shift then
        begin
          Position := 0;
          Key := 0;
        end
        else if [ssShift] = Shift then
        begin
          Back10Button.Click;
          Key := 0;
        end
        else if [] = Shift then
        begin
          BackButton.Click;
          Key := 0;
        end;
      VK_RIGHT:
        if [ssCtrl] = Shift then
        begin
          Position := MediaPlayer.Length - 1;
          Key := 0;
        end
        else if [ssShift] = Shift then
        begin
          Next10Button.Click;
          Key := 0;
        end
        else if [] = Shift then
        begin
          NextButton.Click;
          Key := 0;
        end;
      VK_OEM_4: // [
        begin
          Position := SelStart;
        end;
      VK_OEM_6: // ]
        begin
          Position := SelEnd;
        end;
    end;
  end;
end;

procedure TVideoPlayerForm.Pause;
begin
  if FPlayerReady then
  begin
    MediaPlayer.PauseOnly;
  end;
end;

procedure TVideoPlayerForm.SetSelEnd(Value: Integer);
begin
  if FPlayerReady then
  begin
    Value := Max(0, Min(MediaPlayer.Length, Value));
    FSelEnd := Value;
    if Value = FSelStart then
    begin
      MediaPlayer.EndPos := MediaPlayer.Length
    end
    else
    begin
      MediaPlayer.EndPos := Value;
    end;
  end
  else
  begin
    FSelEnd := 0;
  end;
end;

procedure TVideoPlayerForm.SetSelStart(Value: Integer);
begin
  if FPlayerReady then
  begin
    Value := Max(0, Min(MediaPlayer.Length - 1, Value));

    FSelStart := Value;
    MediaPlayer.StartPos := Value;
    MediaPlayer.Position := Value;
    FOldPosition := Value;
    FPosition := Value;
  end
  else
  begin
    FSelStart := 0;
    FOldPosition := 0;
    FPosition := 0;
  end;
end;

procedure TVideoPlayerForm.TimerTimer(Sender: TObject);
begin
  FPosition := MediaPlayer.Position;
  if FOldPosition <> FPosition then
  begin
    PositionChange;
    FOldPosition := FPosition;
  end;

  PlayButton.Down := MediaPlayer.Mode = mpPlaying;
  Timer.Enabled := MediaPlayer.Mode = mpPlaying;
  PauseButton.Down := MediaPlayer.Mode = mpPaused;
  StopButton.Down := MediaPlayer.Mode = mpStopped;

  if MediaPlayer.Mode = mpStopped then
  begin
    MediaPlayer.StartPos := FSelStart;
    MediaPlayer.EndPos := FSelEnd;
  end;
end;

procedure TVideoPlayerForm.PositionChange;
begin
  if Assigned(FOnPositionChange) then
  begin
    FOnPositionChange(Self);
  end;
end;

procedure TVideoPlayerForm.SetPosition(Value: Integer);
begin
  if FPlayerReady then
  begin
    Value := Max(0, Min(MediaPlayer.Length - 1, Value));

    if FPosition = Value then Exit;

    FPosition := Value;
    MediaPlayer.PauseOnly;
    MediaPlayer.Position := Value;
    PositionChange;
    FOldPosition := Value;
  end
  else
  begin
    FPosition := 0;
    FOldPosition := 0;
  end;
end;

procedure TVideoPlayerForm.SaveSettings(Registry: TRegistry);
begin
  try Registry.WriteInteger('Left', Left); except end;
  try Registry.WriteInteger('Top', Top); except end;
  try Registry.WriteInteger('Width', Width); except end;
  try Registry.WriteInteger('Height', Height); except end;
end;

procedure TVideoPlayerForm.LoadSettings(Registry: TRegistry);
begin
  try Left := Registry.ReadInteger('Left'); except end;
  try Top := Registry.ReadInteger('Top'); except end;
  try Width := Registry.ReadInteger('Width'); except end;
  try Height := Registry.ReadInteger('Height'); except end;
end;

end.
